/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package info.aspirecn.iov.sjjh.util;

import com.alibaba.fastjson.JSON;
import info.aspirecn.iov.sjjh.constant.Constants;
import info.aspirecn.iov.sjjh.constant.ContentType;
import info.aspirecn.iov.sjjh.constant.HttpHeader;
import info.aspirecn.iov.sjjh.entity.Request;
import info.aspirecn.iov.sjjh.entity.Response;
import info.aspirecn.iov.sjjh.enums.Method;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.RequestBody;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 调用示例
 * 请替换APP_KEY,APP_SECRET,HOST,CUSTOM_HEADERS_TO_SIGN_PREFIX为真实配置
 */
public class PostUtil {
	//网关的host地址
	public static String HOST = "https://gateway.logink.cn";


    /**
     * HTTP POST 字符串
     *
     * @throws Exception
     */
    public static Response postString(String URL, String BODY,
                                      String APP_KEY, String APP_SECRET, Integer outTime) throws Exception {
    	String PATH = URL.substring(HOST.length(), URL.length());
        Map<String, String> headers = new HashMap<String, String>();
        //（必填）根据期望的Response内容类型设置
        headers.put(HttpHeader.HTTP_HEADER_ACCEPT, ContentType.CONTENT_TYPE_JSON);
        //（可选）Body MD5,服务端会校验Body内容是否被篡改,建议Body非Form表单时添加此Header
        headers.put(HttpHeader.HTTP_HEADER_CONTENT_MD5, MessageDigestUtil.base64AndMD5(BODY));
        //（POST/PUT请求必选）请求Body内容格式
        headers.put(HttpHeader.HTTP_HEADER_CONTENT_TYPE, ContentType.CONTENT_TYPE_JSON);

        List<String> CUSTOM_HEADERS_TO_SIGN_PREFIX = new ArrayList<String>();

        Request request = new Request(Method.POST_STRING, HOST,
        		PATH, APP_KEY, APP_SECRET, outTime);
        request.setHeaders(headers);
        request.setSignHeaderPrefixList(CUSTOM_HEADERS_TO_SIGN_PREFIX);

        //（非必填）根据api描述，如果有需要传递的参数，设置请求的query
        Map<String, String> querys = new HashMap<String, String>();
        request.setQuerys(querys);
        request.setStringBody(BODY);

        //调用服务端
        Response response = Client.execute(request);
        return response;
    }

    public static okhttp3.Response postString(String url, String body, Integer outTime) throws Exception {
        MediaType mediaType = MediaType.parse(Constants.APPLICATION_JSON_UTF8_VALUE);
        RequestBody requestBody = RequestBody.create(mediaType, body);
        okhttp3.Request okRequest = new okhttp3.Request.Builder()
                .post(requestBody)
                .url(url)
                .build();
        OkHttpClient okHttpClient = new OkHttpClient.Builder().build();
        OkHttpClient client = okHttpClient.newBuilder()
                .connectTimeout(outTime, TimeUnit.MILLISECONDS)
                .readTimeout(outTime, TimeUnit.MILLISECONDS)
                .writeTimeout(outTime, TimeUnit.MILLISECONDS)
                .build();
        return  client.newCall(okRequest).execute();
    }

}
